@@ -1,6 +1,6 @@
 ###############################################################################
-# Copyright 2017 - Climate Research Division
-#                  Environment and Climate Change Canada
+# Copyright 2017-2021 - Climate Research Division
+#                       Environment and Climate Change Canada
 #
 # This file is part of the "fstd2nc" package.
 #
@@ -23,7 +23,7 @@
 Functionality for converting between FSTD and netCDF files.
 """
 
-__version__ = "0.20210305.1"
+__version__ = "0.20211013.0"
 
 
 # Check for bundled rpnpy package.
@@ -50,13 +50,14 @@ from fstd2nc.mixins.xycoords import XYCoords
 from fstd2nc.mixins.misc import NoNK
 from fstd2nc.mixins.filter import FilterRecords
 from fstd2nc.mixins.removestuff import RemoveStuff
+from fstd2nc.mixins.interp import Interp
 from fstd2nc.mixins.pruneaxes import PruneAxes
 from fstd2nc.mixins.netcdf import netCDF_Atts, netCDF_IO
 from fstd2nc.mixins.compat import FSTD_Compat
-from fstd2nc.mixins.extern import Extern
+from fstd2nc.mixins.extern import ExternInput, ExternOutput
 from fstd2nc.mixins.iter import Iter
 
-class Buffer (Iter,Extern,FSTD_Compat,netCDF_IO,netCDF_Atts,PruneAxes,RemoveStuff,FilterRecords,NoNK,XYCoords,VCoords,Sfc_Codes,VarDict,Series,Ensembles,Dates,Masks,ASCII,SelectVars):
+class Buffer (Iter,ExternOutput,FSTD_Compat,netCDF_IO,netCDF_Atts,PruneAxes,Interp,RemoveStuff,FilterRecords,NoNK,XYCoords,VCoords,Sfc_Codes,VarDict,Series,Ensembles,Dates,Masks,ASCII,SelectVars,ExternInput):
   """
   High-level interface for FSTD data, to treat it as multi-dimensional arrays.
   Contains logic for dealing with most of the common FSTD file conventions.
