@@ -1,6 +1,6 @@
 ###############################################################################
-# Copyright 2017 - Climate Research Division
-#                  Environment and Climate Change Canada
+# Copyright 2017-2021 - Climate Research Division
+#                       Environment and Climate Change Canada
 #
 # This file is part of the "fstd2nc" package.
 #
@@ -32,7 +32,7 @@ class netCDF_Atts (BufferBase):
     parser.add_argument('--metadata-file', type=argparse.FileType('r'), action='append', help=_('Use metadata from the specified file.  You can repeat this option multiple times to build metadata from different sources.'))
     parser.add_argument('--rename', metavar="OLDNAME=NEWNAME,...", help=_('Apply the specified name changes to the variables.'))
     parser.add_argument('--conventions', default='CF-1.6', help=_('Set the "Conventions" attribute for the netCDF file.  Default is "%(default)s".  Note that this has no effect on the structure of the file.'))
-    parser.add_argument('--no-conventions', action='store_true', help=_('Omit the "Conventions" attribute from the netCDF file entirely.'))
+    parser.add_argument('--no-conventions', action='store_true', help=_('Omit the "Conventions" attribute from the netCDF file entirely.  This can help for netCDF tools that have trouble recognizing the CF conventions encoded in the file.'))
   @classmethod
   def _check_args (cls, parser, args):
     super(netCDF_Atts,cls)._check_args(parser,args)
@@ -53,6 +53,8 @@ class netCDF_Atts (BufferBase):
        Note that this has no effect on the structure of the file.
     no_conventions : bool, optional
        Omit the "Conventions" attribute from the netCDF file entirely.
+       This can help for netCDF tools that have trouble recognizing the CF
+       conventions encoded in the file.
     """
     try:
       import ConfigParser
@@ -122,13 +124,11 @@ class netCDF_Atts (BufferBase):
         renames[varname] = atts.pop('rename')
 
     # Apply the user-supplied metadata.
-    for var in self._varlist:
-      # Add extra metadata provided by the user?
-      if var.name in self._metadata:
-        var.atts.update(self._metadata[var.name])
-
-    # Apply renames.
     for obj in self._iter_objects():
+      # Add extra metadata provided by the user?
+      if obj.name in self._metadata:
+        obj.atts.update(self._metadata[obj.name])
+      # Apply renames.
       if obj.name in renames:
         obj.name = renames[obj.name]
 
@@ -198,7 +198,7 @@ class netCDF_IO (BufferBase):
         var.array = np.array(var.array.tolist())
         units = '%s since %s'%(self._time_units, reference_date or var.array.reshape(-1)[0])
         var.atts.update(units=units, calendar='gregorian')
-        var.array = np.asarray(date2num(var.array,units=units))
+        var.array = np.asarray(date2num(var.array,units=units), dtype='double')
 
     for obj in self._iter_objects():
       # Encode the attributes so they're ready for writing to netCDF.
