@@ -1,6 +1,6 @@
 ###############################################################################
-# Copyright 2017 - Climate Research Division
-#                  Environment and Climate Change Canada
+# Copyright 2017-2021 - Climate Research Division
+#                       Environment and Climate Change Canada
 #
 # This file is part of the "fstd2nc" package.
 #
@@ -50,14 +50,14 @@ class VCoords (BufferBase):
     parser.add_argument('--strict-vcoord-match', action='store_true', help=_("Require the IP1/IP2/IP3 parameters of the vertical coordinate to match the IG1/IG2/IG3 paramters of the field in order to be used.  The default behaviour is to use the vertical record anyway if it's the only one in the file."))
     group = parser.add_mutually_exclusive_group()
     group.add_argument('--diag-as-model-level', action='store_true', help=_("Treat diagnostic (near-surface) data as model level '1.0'.  This is the default behaviour."))
-    group.add_argument('--split-diag-level', action='store_true', help=_("Put the diagnostic (near-surface) data in a separate variable, away from the 3D model output.  Suffices will be added to distinguish the different types of levels (e.g. _vgrid4 and _vgrid5 for diagnostic height and hybrid levels respectively)."))
+    group.add_argument('--split-diag-level', action='store_true', help=_("Put the diagnostic (near-surface) data in a separate variable, away from the 3D model output.  Suffices will be added to distinguish the different types of levels (i.e. _diag_level and _model_levels for diagnostic height and hybrid levels respectively)."))
     group.add_argument('--ignore-diag-level', action='store_true', help=_("Ignore data on diagnostic (near-surface) height."))
     parser.add_argument('--thermodynamic-levels', action='store_true', help=_("Only convert data that's on 'thermodynamic' vertical levels, ignoring data on 'momentum' levels."))
 
   # Need to extend _headers_dtype before __init__.
   def __new__ (cls, *args, **kwargs):
     obj = super(VCoords,cls).__new__(cls, *args, **kwargs)
-    obj._headers_dtype = obj._headers_dtype + [('level','float32'),('kind','int32')]
+    obj._headers_dtype = obj._headers_dtype + [('level','float32'),('kind','int32'),('level_descr','|S20'),('special_level','|S20')]
     return obj
 
   def __init__ (self, *args, **kwargs):
@@ -73,7 +73,7 @@ class VCoords (BufferBase):
     split_diag_level : bool, optional
         Put the diagnostic (near-surface) data in a separate variable, away
         from the 3D model output.  Suffices will be added to distinguish
-        the different types of levels (e.g. _vgrid4 and _vgrid5 for
+        the different types of levels (i.e. _diag_level and _model_levels for
         diagnostic height and hybrid levels respectively).
     ignore_diag_level : bool, optional
         Ignore data on diagnostic (near-surface) height.
@@ -95,18 +95,20 @@ class VCoords (BufferBase):
     # Use decoded IP1 values as the vertical axis.
     self._outer_axes = ('level',) + self._outer_axes
     # Tell the decoder not to process vertical records as variables.
-    self._meta_records = self._meta_records + ('!!',)
-    self._maybe_meta_records = self._maybe_meta_records + ('HY',)
+    self._meta_records = self._meta_records + (b'!!',b'!!SF')
+    self._maybe_meta_records = self._maybe_meta_records + (b'HY',)
     super(VCoords,self).__init__(*args,**kwargs)
     # Don't group records across different level 'kind'.
     # (otherwise can't create a coherent vertical axis).
     self._var_id = self._var_id + ('kind',)
-    self._human_var_id = self._human_var_id + ('vgrid%(kind)s',)
+    self._human_var_id = self._human_var_id + ('%(special_level)s','%(level_descr)s','vgrid%(kind)s')
 
     # Pre-scan the raw headers for special vertical records.
     # (these aren't available in the data stream, because we told the decoder
     # to ignore them).
     vrecs = OrderedDict()
+    # Keep track of any diagnostic levels, e.g. from GEM.
+    diag_ip1 = []
     for vcoord_nomvar in ('HY','!!'):
       for handle in fstinl(self._meta_funit, nomvar=vcoord_nomvar):
         header = fstprm(handle)
@@ -115,6 +117,19 @@ class VCoords (BufferBase):
         if header['nomvar'].strip() == 'HY': key = 'HY'
         if key in vrecs: continue
         vrecs[key] = header
+        # Extract diagnostic-level value from vertical coordinate.
+        if key == 'HY': continue  # not applicable for HY coordinates.
+        prm = fstluk(header,rank=3)
+        try:
+          vgd_id = vgd_fromlist(prm['d'])
+          diag_ip1.append(vgd_get(vgd_id,'DIPT'))
+          diag_ip1.append(vgd_get(vgd_id,'DIPM'))
+        except (KeyError,VGDError):
+          warn(_("Unable to parse diagnostic levels from the vertical coordinate"))
+          continue
+        vgd_free (vgd_id)
+        # Done exracting diagnostic levels.
+
     self._vrecs = vrecs
 
     # Check if there are multiple '!!' records that all contain
@@ -144,25 +159,14 @@ class VCoords (BufferBase):
     # Start by treating diagnostic level as model level, then
     # revert this later if it ends up not working.
     if not self._split_diag_level:
-      for key, header in self._vrecs.items():
-        if key == 'HY': continue  # not applicable for HY coordinates.
-        prm = fstluk(header,rank=3)
-        vgd_id = vgd_fromlist(prm['d'])
-        try:
-          ip1_t = vgd_get(vgd_id,'DIPT')
-          ip1_m = vgd_get(vgd_id,'DIPM')
-          mask = (fields['ip1'] == ip1_t) | (fields['ip1'] == ip1_m)
-          if len(self._vrecs) > 1 or self._strict_vcoord_match:
-            mask &= (fields['ig1'] == prm['ip1']) & (fields['ig2'] == prm['ip2'])
+      for ip1_val in diag_ip1:
+          mask = (fields['ip1'] == ip1_val)
           if self._ignore_diag_level:
             fields['dltf'] |= mask
           else:
             #fields['ip1'][mask] = 93423264
             fields['level'][mask] = 1.0
             fields['kind'][mask] = 5
-        except (KeyError,VGDError):
-          warn(_("Unable to parse diagnostic levels from the vertical coordinate"))
-        vgd_free (vgd_id)
 
     # Keep only "thermodynamic" levels?
     # (Discard any levels considered to be "momentum" levels).
@@ -181,6 +185,24 @@ class VCoords (BufferBase):
           warn(_("Unable to parse momentum levels from the vertical coordinate"))
         vgd_free (vgd_id)
 
+    # Apply special labels for some levels.
+    # Used to describe a variable when it's split into multiple coordinate types.
+    fields['level_descr'][:] = 'unknown_levels'
+    fields['level_descr'][fields['kind'] == 0] = 'depth_levels'
+    fields['level_descr'][fields['kind'] == 1] = 'model_levels'
+    fields['level_descr'][fields['kind'] == 2] = 'pressure_levels'
+    fields['level_descr'][fields['kind'] == 3] = 'generic_levels'
+    fields['level_descr'][fields['kind'] == 4] = 'height_levels'
+    fields['level_descr'][fields['kind'] == 5] = 'model_levels'
+    fields['level_descr'][fields['kind'] == 21] = 'model_levels'
+    fields['special_level'][:] = fields['level_descr'][:]
+    # Discern ocean depths from ocean bottom.
+    fields['special_level'][fields['kind'] == 0] = 'depth_levels'
+    fields['special_level'][(fields['kind'] == 1) & (fields['level'] == 1.0)] = 'bottom_level'
+    # Discern atmospheric levels from diagnostic levels.
+    for ip1_val in diag_ip1:
+      fields['special_level'][fields['ip1'] == ip1_val] = 'diag_level'
+
   def _makevars (self):
     from fstd2nc.mixins import _var_type, _axis_type, _dim_type
     from collections import OrderedDict
@@ -285,7 +307,11 @@ class VCoords (BufferBase):
             # Add type-specific metadata.
             if header['nomvar'].strip() == '!!':
               # Get A and B info.
-              vgd_id = vgd_fromlist(fstluk(header,rank=3)['d'])
+              try:
+                vgd_id = vgd_fromlist(fstluk(header,rank=3)['d'])
+              except VGDError:
+                warn (_("Problem decoding !! record."))
+                continue
               # Partial definition of a/b coordinates for formula_terms reference.
               coordA = _var_type('a', {}, [new_axis], None)
               coordB = _var_type('b', {}, [new_axis], None)
@@ -408,6 +434,14 @@ class VCoords (BufferBase):
       for var in self._varlist:
         if not hasattr(var,'record_id'): continue
         valid_records = var.record_id[var.record_id>=0]
+        # Ugly hack to find all related records.
+        # There may be duplicate records that aren't in this set, but would
+        # need to be transformed identically.
+        # E.g., if have multiple copies of the diagnostic level, then we want
+        # them all to stay equivalent or suddenly get multiple versions of the
+        # field showing up on the 'rerun' iteration.
+        nomvar = self._headers['nomvar'][valid_records][0]
+        valid_records = np.where(self._headers['nomvar'] == nomvar)[0]
         ip1 = self._headers['ip1'][valid_records]
         decoded = np.concatenate(decode_ip1(ip1))
         if not any(decoded['kind']==4): continue
