@@ -1,6 +1,6 @@
 ###############################################################################
-# Copyright 2017 - Climate Research Division
-#                  Environment and Climate Change Canada
+# Copyright 2017-2021 - Climate Research Division
+#                       Environment and Climate Change Canada
 #
 # This file is part of the "fstd2nc" package.
 #
@@ -116,11 +116,13 @@ class LatLon(GridMap):
     self._lonatts['long_name'] = 'longitude'
     self._lonatts['standard_name'] = 'longitude'
     self._lonatts['units'] = 'degrees_east'
+    self._lonatts['axis'] = 'X'
     self._latarray = ll['lat'][0,:]
     self._latatts = OrderedDict()
     self._latatts['long_name'] = 'latitude'
     self._latatts['standard_name'] = 'latitude'
     self._latatts['units'] = 'degrees_north'
+    self._latatts['axis'] = 'Y'
     self.lon = _axis_type('lon',self._lonatts,self._lonarray)
     self.lat = _axis_type('lat',self._latatts,self._latarray)
     self.gridaxes = [self.lat,self.lon]
@@ -197,11 +199,13 @@ class RotLatLon(GridMap):
     self._lonatts['long_name'] = 'longitude'
     self._lonatts['standard_name'] = 'longitude'
     self._lonatts['units'] = 'degrees_east'
+    self._lonatts['axis'] = 'X'
     self._latarray = ll['lat'].transpose() # Switch from Fortran to C order.
     self._latatts = OrderedDict()
     self._latatts['long_name'] = 'latitude'
     self._latatts['standard_name'] = 'latitude'
     self._latatts['units'] = 'degrees_north'
+    self._latatts['axis'] = 'Y'
     self.lon = _var_type('lon',self._lonatts,self.gridaxes,self._lonarray)
     self.lat = _var_type('lat',self._latatts,self.gridaxes,self._latarray)
     return (self.xaxis, self.yaxis, self.gridaxes, self.lon, self.lat)
@@ -262,11 +266,13 @@ class PolarStereo(GridMap):
     self._lonatts['long_name'] = 'longitude'
     self._lonatts['standard_name'] = 'longitude'
     self._lonatts['units'] = 'degrees_east'
+    self._lonatts['axis'] = 'X'
     self._latarray = ll['lat'].transpose() # Switch from Fortran to C order.
     self._latatts = OrderedDict()
     self._latatts['long_name'] = 'latitude'
     self._latatts['standard_name'] = 'latitude'
     self._latatts['units'] = 'degrees_north'
+    self._latatts['axis'] = 'Y'
     xy = gdxyfll(self._grd['id'],ll['lat'],ll['lon'])
     # Scale grid coordinates back to actual coordinates in projection plane   
     self._ax = ( np.rint(xy['x'][:,0]) - 1) * self._res   # metres
@@ -304,10 +310,10 @@ class PolarStereo(GridMap):
 class XYCoords (BufferBase):
   # Special records that contain coordinate info.
   # We don't want to output these directly as variables, need to decode first.
-  _xycoord_nomvars = ('^^','>>','^>')
+  _xycoord_nomvars = (b'^^',b'>>',b'^>')
   # Grids that can be read directly from '^^','>>' records, instead of going
   # through ezqkdef (in fact, may crash ezqkdef if you try decoding them).
-  _direct_grids = ('X','Y','T','+')
+  _direct_grids = ('X','Y','T','+','O')
 
   @classmethod
   def _cmdline_args (cls, parser):
@@ -432,10 +438,12 @@ class XYCoords (BufferBase):
           latatts['long_name'] = 'latitude'
           latatts['standard_name'] = 'latitude'
           latatts['units'] = 'degrees_north'
+          latatts['axis'] = 'Y'
           lonatts = OrderedDict()
           lonatts['long_name'] = 'longitude'
           lonatts['standard_name'] = 'longitude'
           lonatts['units'] = 'degrees_east'
+          lonatts['axis'] = 'X'
 
           latarray = lonarray = None
           try:
@@ -602,4 +610,5 @@ class XYCoords (BufferBase):
     # coordinates.  In this case, include the lat/lon as variables.
     if len(self._varlist) == 0 and len(lats) == 1 and len(lons) == 1:
       self._varlist = [list(lats.values())[0], list(lons.values())[0]]
-
+      # Add grid mapping info.
+      self._varlist.extend(gridmaps.values())
